#define HASSNPRINTF 1
#ifndef __P
#define __P(proto) proto
#endif

/***************************************/
/*      START OF CONFIGURABLE          */
/*           OPTIONS                   */
/***************************************/
#define DISCLAIMER 1        /* CHANGE TO 1 FOR DISCLAIMER, 0 FOR NO DISCLAIMER.         */
#define MAX_DISCL_SZ 2500   /* Maximum size of disclaimer.                           */
#define BND_SZ 80           /* MAX Boundary size.                                    */
/***************************************/
/*      END OF CONFIGURABLE            */
/*           OPTIONS                   */
/***************************************/
#include <fcntl.h>
#ifdef linux
#include <getopt.h>
#endif 
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sysexits.h> 
#include <termios.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include "libmilter/mfapi.h"

#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>


/*
 * Remove the *NSPAM* before mailing.
 * Al Smith <Al.Smith@*NSPAM*aeschi.ch.eu.org>  May 9 2000
 *
 * Enhancements submitted by Sven Nielsen <dalvenjah@*NSPAM*dal.net>  May 13 2000
 * Inline rejection suggested by Paul Yeh <Paul.Yeh@*NSPAM*abc.com>   May 24 2000
 * Standard disclaimer by  Wouter Wijker<w.wijker@*NSPAM*cvg.nl>  Jul 12 2000
 * Standard disclaimer bug-fixes Wouter Wijker<w.wijker@*NSPAM*cvg.nl> SEP 8 2000
 * Standard disclaimer for sendmail 8.11.1  06-10-2000  **W.Wijker**
 *
 * Please note that this code requires sendmail-8.11.1 or newer.
 *
 * This code is distributed under the GNU General Public License.
 * Share and enjoy.
 */

/*
 * HISTORY (Andrew from ledge.co.za)
 * See Changelog file for more ...
 * Dec  7  2004 -- Removed everything except the disclaimer handling.  Fixed
 *                 handling of multiple domains, read disclaimer from file,
 *                 better command line options.  This is a big downgrade in
 *                 functionality, but it's what I need today - spamassassin is
 *                 doing an ok job on the spam already...
 * HISTORY (*W.Wijker*)
 * Feb 25  2004 -- Do not accept mails if subject-field is omitted from the mailheader. 
 * Feb 24  2004 -- WINDOWS-1252?Q? spam-subjects also tested.
 * Feb 20  2004 -- Zip and dll extensions blocked. (MyDoom/NetSky)
 * Feb 20  2004 -- Discard E-mail silently or Reject-message to possibly wrong sender?(MyDoom/NetSky)
 * Feb 11  2004 -- Spamwords added. Where does it stop?
 * Jan 21  2004 -- ISO-8859-1?B? spam-subjects blocked.
 * Dec 29  2003 -- Anti Spam-enhancements.
 * Dec 19  2003 -- Disclaimer split in several languages. (Prevents compiler warning).
 * Dec 19  2003 -- Anti-spam capabilities added by testing the Subject-line.
 *                 NOT intended to replace spamassasin or any dnsrbl. Just additional.
 * Dec 18  2003 -- Changes to detect_header() function. Different way of determining
 *                 extension of the attachment.
 * Dec 17  2003 -- Trace functionality for debugging purposes added.
 * Dec 15  2003 -- Multiple domain-names allowed for adding disclaimer.
 *                 Suggestion from Steven Lockhart [stevel@*NSPAM*hugall.com.au] who has
 *                 multiple domains on one mail-server.
 * Dec 13  2003 -- PARANOID mode added. (Just one and only one extension allowed)
 * Nov 21  2003 -- Remarks of Milan Kerslager implemented. (finally)
 * Sep  3  2003 -- Added "zi" extension to also block variant of Sobig virus.
 * Sep  3  2003 -- Re-positioning of #defines to compile without DISCLAIMER usage.
 *                 Thanks: Milan Kerslager<milan.kerslager@*NSPAM*pslib.cz>
 * Aug 19  2003 -- QAD-patch to test Lotus-Notes client.
 * Mar 20  2003 -- Synchronised with vbsfilter 1.11.a
 * Jul  1  2002 -- Improved detection of Attachment-name.
 * Jun 26  2002 -- Unquoted boundary in Outlook-Express 5.50.4133.2400
 *              -- Addition of mpg, mpe, eml, mp3 and wav to list of extensions.
 * Jun 25  2002 -- Synchronized this version with vbsfilter 1.9
 *                 Modified version of detect_header.
 *                 Included the suggestions from :
 *
 * Ross Bergman <rbergman@*NSPAM*vividusa.com> noted that some malformed endings,
 * for example "foo.exe.". Also that a file can have a CLSID as a file
 * extension, again hidden by windows, and Windows will use the CLSID to
 * determine how to open the attachment.
 *
 * Updated vbsfilter to detect uuencoded attachments.
 * Thanks to Matthew Wong <matthew_wong@*NSPAM*aelhk.com.hk> for noting
 * that the "MyParty" virus spreads itself in this way.
 *
 * Various fixes for the case where headers and/or body are empty.
 * Patch sent in by Sergiy Zhuk <serge@*NSPAM*yahoo-inc.com>
 *
 * Update "dangerous extension" list.
 * Noted by David F. Russell <David.F.Russell@*NSPAM*ncmail.net>
 *
 * Mar 05  2002 -- Correction for "filename="=?iso-8859-1?Q?Install=5Fclient.bat?="
 *                 Content-type: application/octect/stream;
 *                 Means that the fishy extension is not in the last 4 chars.
 * Dec 10  2001 -- Block SCR files.
 * Nov  6  2001 -- Block HTA en REG ivm JS/Seeker.i virus.
 * Nov  1  2001 -- Block MPE
 * Okt  24 2001 -- Block pif usage.
 * Sept 19 2001 -- Block eml usage. (NIMDA virus)
 * Sept 17 2001 -- Block mpg usage. (too much net traffic)
 * Aug  6  2001 -- Added extensions to prevent SirCamm virus. (PIF LNK)
 * May  15 2001 -- Attempt to prevent memory leakage by re-construct
 *                 of realloc() phrasing.
 * Apr  9  2001 -- Prevent double execution of replacebody() when both
 *                 exe->txt and disclaimer are choosen.
 * Mar  20 2001 -- Automatic boundary detection and support of
 *                 Netscape-messenger. (MIME) ( Suggestion Jai Lamerton)
 *                 Disclaimer tested for: Netscape Communicator 4.76
 *                                        OUTLOOK-97
 *                                        OUTLOOK-Express
 *                                        OUTLOOK-2000
 *                                        Eudora 5.0.2
 * Okt  6  2000 -- Changes for sendmail 8.11.1
 * Okt  3  2000 -- Changing MIME exe attachements to txt.
 * SEPT 8  2000 -- UPDATES to 1.3 and bug-fixing Disclaimer.
 * July 26 2000 -- Addition of Legal Disclaimer functionality. (W Wijker)
 * June 20 2000 -- vbsfilter-1.2
 * Add recognition of .SHS attachments.
 *
 * May 24 2000 -- vbsfilter-1.1
 * Add recognition of inline VBS attachments.
 * Libmilter doesn't let us modify the headers, so such mails
 * must be rejected.
 *
 * May  9 2000 -- vbsfilter-1.0
 * Filter incoming visual basic script attachments into text attachments,
 * by changing the .vbs filename suffix into .txt.
 */

#if DISCLAIMER == 1
/*****************************************************************************/
/* REPLACE MY LEGAL RUBBISH BELOW WITH YOUR OWN LEGAL RUBBISH FORCED         */
/* UPON YOU BY YOUR OWN WELL PAYED LAWYERS.                                  */
/*                                                                           */
/*****************************************************************************/
char *disclaimertext = NULL;
 

#endif

/*****************************************************************************/
/* REPLACE IN BELOW LINE THE "cvg.nl" BY THE DOMAIN YOU WANT THE DISCLAIMER  */
/* USED FOR.                                                                 */
/*****************************************************************************/
#define MAXDOMAINS 16
int ourdomainscount = 0;
char *ourdomains[MAXDOMAINS];
char *disclaimer = NULL;

/******************************************************************************/
/********          END REPLACEMENTS    ****************************************/
/******************************************************************************/

struct mlfiPriv {
	int   len;
	u_char *body;
        int    internal_sender;
#if DISCLAIMER == 1
        int    txtdiscl_inserted;
        int    htmldiscl_inserted;
        int    allow;
        int    htmlallow;
        char   boundary[BND_SZ];
        int    msgtype;
#endif
};

#if DISCLAIMER == 1
int insert_disclaimer(struct mlfiPriv *priv);
void initDisclaimer(SMFICTX *ctx, char *envfrom);
void detectMsgFormat(SMFICTX *ctx,char *headerf,char *headerv);
#endif

void detectSender(SMFICTX *ctx, char *envfrom);


#define MLFIPRIV ((struct mlfiPriv *) smfi_getpriv(ctx))

extern sfsistat mlfi_cleanup(SMFICTX *, sfsistat);

/******************************************************/
static int nocase_strncmp(const char *s1, const char *s2, int len) {
        register int n = len;

        for (; n; s1++, s2++, n--) {
                if (tolower(*s1) != tolower(*s2)) return (1);
        }
        return(n > 0);
}
/******************************************************/
static char * nocase_strstr(const char *s1, const char *s2) {
	const char *p, *q, *r;
	char x, y;
	for(p = s1; *p; p++) {
		for(q = s2, r = p; *q && *r; q++, r++) {
			x = (isalpha((int) *q)) ? tolower(*q) : *q;
			y = (isalpha((int) *r)) ? tolower(*r) : *r;

			if (x != y) break;
		}
		if (!*q) {
			return((char *) p);
		} else if (!*r) {
			return(NULL);
		}
	}
	return(NULL);
}
/******************************************************/
sfsistat mlfi_envfrom(SMFICTX *ctx, char **envfrom) {
	struct mlfiPriv *priv;
	priv = (struct mlfiPriv *)malloc(sizeof *priv);
	if (!priv) {
		/* can't accept this message right now */
		return SMFIS_TEMPFAIL;
	}
        memset(priv,0x00,sizeof(priv));
	priv->body = NULL;
	priv->len  = 0;
	smfi_setpriv(ctx, priv);
        detectSender(ctx, *envfrom);
#if DISCLAIMER==1
        initDisclaimer(ctx,*envfrom);
#endif
	/* save the private data */
	/* continue processing */
	return SMFIS_CONTINUE;
}
/******************************************************/
sfsistat mlfi_header(SMFICTX *ctx, char *headerf, char *headerv) {
	//struct mlfiPriv *priv = MLFIPRIV;

#if DISCLAIMER==1      
        detectMsgFormat(ctx,headerf,headerv);
#endif
	return SMFIS_CONTINUE;
}
/******************************************************/
sfsistat mlfi_eoh(SMFICTX *ctx) {
        return SMFIS_CONTINUE;
}
/******************************************************/
sfsistat mlfi_body(SMFICTX *ctx, u_char *bodyp, size_t bodylen) {
	struct mlfiPriv *priv = MLFIPRIV;
        u_char *memp;
        memp = (u_char *)realloc(priv->body, priv->len + bodylen +1);
	if (!memp) {
		/* can't accept this message right now */
		// return SMFIS_TEMPFAIL;
		return SMFIS_CONTINUE;
	}
        priv->body=memp;
	memcpy((u_char*)(priv->body+priv->len), bodyp, bodylen);
        priv->len += bodylen;
        priv->body[priv->len]= '\0';
	/* continue processing */
	return SMFIS_CONTINUE;
}
/******************************************************/
sfsistat mlfi_eom(SMFICTX *ctx) {
	struct mlfiPriv *priv = MLFIPRIV;
#if DISCLAIMER == 1
        int replaceanyway;
        replaceanyway=0;
#endif
        if (!priv->body) {
                return mlfi_cleanup(ctx, SMFIS_CONTINUE);
        }

#if DISCLAIMER == 1
        if (insert_disclaimer(priv) || replaceanyway)
           { 
             if (priv->body)
                {
	         smfi_replacebody(ctx, priv->body, priv->len);
                }
           }
#endif
	return mlfi_cleanup(ctx, SMFIS_CONTINUE);
}

/******************************************************/
sfsistat mlfi_close(SMFICTX *ctx) {
	return mlfi_cleanup(ctx, SMFIS_ACCEPT);
} 
/******************************************************/
sfsistat mlfi_abort(SMFICTX *ctx) {
	return mlfi_cleanup(ctx, SMFIS_CONTINUE);
}
/******************************************************/
sfsistat mlfi_cleanup(SMFICTX *ctx, sfsistat rc) {
	struct mlfiPriv *priv = MLFIPRIV;
	if (priv) {
		if (priv->body) free(priv->body);
		free(priv);
		smfi_setpriv(ctx, NULL);
	}
	return(rc);
}
/******************************************************/
struct smfiDesc smfilter = {
	"disclaimermilter",	/* filter name */
	SMFI_VERSION,	/* version code -- do not change */
	SMFIF_ADDHDRS | SMFIF_CHGBODY ,	/* flags */
	NULL,		/* connection info filter */
	NULL,		/* SMTP HELO command filter */
	mlfi_envfrom,	/* envelope sender filter */
	NULL,		/* envelope recipient filter */
	mlfi_header,	/* header filter */
	mlfi_eoh,	/* end of header */
	mlfi_body,	/* body block filter */
	mlfi_eom,	/* end of message */
	mlfi_abort,	/* message aborted */
	mlfi_close	/* connection cleanup */
};

void usage(char *message)
{
	fprintf(stderr,"Usage: disclaimermilter <options>\n"
		"  http://www.ledge.co.za/software/disclaimermilter/\n"
		"Options:\n"
		"  -p socket    milter socket to create\n"
		"  -d domain    domain(s) to add disclaimer for (can repeat)\n"
		"  -m message   file name from which to read the disclaimer\n"
		"\n");
	if (message) {
		fprintf(stderr,"%s\n",message);
	}
	exit(EX_USAGE);
}

/* Read a file, and return a string */
char *readfiletostring(char *filename, int maxsize)
{
	FILE *f;
	struct stat filestat;
	int filesize,rc;
	char *message;
	f=fopen(filename,"r");
	if (f==NULL) {
		perror("fopen");
		exit(1);
	}
	if (fstat(fileno(f),&filestat) == -1) {
		perror("fstat");
		exit(1);
	}
	if ( filestat.st_size > MAX_DISCL_SZ) {
		fprintf(stderr,"Disclaimer larger than %d bytes\n",maxsize);
		exit(1);
	}
	filesize = filestat.st_size;
	message = malloc(filesize+1);
	if (message==NULL) {
		fprintf(stderr,"Out of memory\n");
		exit(1);
	}
	message[filesize]=0;
	rc = fread(message,1,filesize,f);
	if (rc != filesize) {
		perror("fread failed");
		exit(1);
	}
	fclose(f);

	return message;
}

/******************************************************/
int main(int argc, char *argv[]) {
	int c, fd;
	const char *args = "p:d:m:";
	/* Process command line options */
	opterr = 0;
	while ((c = getopt(argc, argv, args)) != -1) {
		switch (c) {
		  case 'd':
			if (optarg == NULL || *optarg == '\0') {
				usage("-d requires a domain");
			}
			if (ourdomainscount<MAXDOMAINS) {
				ourdomains[ourdomainscount++] = strdup(optarg);
			}
			else {
				usage("too many -d options");
			}
			break;

		  case 'm':
			if (optarg == NULL || *optarg == '\0') {
				usage("-m requires the name of a message file");
			}
			disclaimertext = readfiletostring(optarg,MAX_DISCL_SZ);
			break;

		  case 'p':
			if (optarg == NULL || *optarg == '\0') {
				usage("-p requires a socket name");
			}
			(void) smfi_setconn(optarg);
			break;
		  default:
			usage("Invalid option given");
		}
	}
	if (disclaimertext == NULL || ourdomainscount == 0) {
		usage("You need to supply a disclaimer and at least one domain");
		exit(1);
	}

	if (smfi_register(smfilter) == MI_FAILURE) {
		fprintf(stderr, "smfi_register failed\n");
		exit(EX_UNAVAILABLE);
	}

	if (fork() == 0) { /*Child process */
                close(STDIN_FILENO);
                close(STDOUT_FILENO);
                close(STDERR_FILENO);
                fd=open("dev/tty",O_RDWR);
                if (fd>=0)
                   { ioctl(fd,TIOCNOTTY,0);
                     close(fd);
                   }
		return smfi_main();  /* libmilter/main.c */
	} 
	return 0;
}
/********************************************************/
void detectSender(SMFICTX *ctx, char *envfrom)
 {
  struct mlfiPriv *priv = MLFIPRIV;  
  priv->internal_sender = 0;
  int i;

  for ( i=0 ; i<ourdomainscount ; i++) {
    if ( ourdomains[i] && nocase_strstr(envfrom,ourdomains[i]) )
     { /* Internal sender from standard domain.
        * Add disclaimer.
        */
       priv->internal_sender = 1;
     }
 }
}
/********************************/
#if DISCLAIMER==1
int insert_disclaimer(struct mlfiPriv *priv)
   {
    /* Insertion of disclaimer in the text/html and text/plain segments
     * of an E-mail message.
     * The Html-message is minimized in font.
     */
    int RC;
    char *end_boundary;
    char *txtplain_p;
    char *txthtml_p;
    u_char *memp;
    char BOUNDARY[BND_SZ];
    char BOUNDARY2[BND_SZ];
    char DISCL2[MAX_DISCL_SZ+100];
    char *tmp_p;
    char *p3, *p4, *p5;
    RC=0;
    memp = (u_char *)realloc(priv->body, priv->len + 5*strlen(disclaimertext));
    if (!memp) {
         /*  Short of memory
          *  Return without destroying priv->body. (18-5-2001)
          */
           return RC;
	}
    priv->body=memp;
    if (priv->internal_sender && priv->allow && !priv->txtdiscl_inserted)
       { /* Insertion into first part of multipart message.
          */
        txtplain_p=(char *)priv->body;
        txtplain_p=(char *)nocase_strstr(txtplain_p,"Content-Type: text/plain");
        if (strlen((char *)priv->boundary))
           { /* Prefix obtained boundary  by "--"
              */
             memset(BOUNDARY,0X00,BND_SZ);
             strcpy(BOUNDARY,"--");
             strcat(BOUNDARY,priv->boundary);
           }
        if (strstr(BOUNDARY,"--=_mixed "))
           { /* Lotus Notes client. Separate boundaries for individual parts.
              * =_mixed 23242424 en =_alternative 23242425
              * Test both and alternative boundary first.
              * Boundaries differ in code also. Limit to 7 positions.
              */
              strcpy(BOUNDARY2,BOUNDARY);
              memset(BOUNDARY,0X00,BND_SZ);
              strcpy(BOUNDARY,"--=_alternative ");
              strncat(BOUNDARY,(char *)&BOUNDARY2[10],7);
          }
        else
          {
             if (strstr(BOUNDARY,"--=_alternative "))
                {
                  memset(BOUNDARY2,0X00,BND_SZ);
                  strcpy(BOUNDARY2,"--=_mixed ");
                  strncat(BOUNDARY2,(char *)&BOUNDARY[16],7);
                }
          }
        if (txtplain_p!=(char *)NULL)
           { /* First segment of message 
              */
             txtplain_p+=13;         
             end_boundary=(char *)strstr(txtplain_p, BOUNDARY);
             if (end_boundary!=(char *)NULL)
                { /* Insert the disclaimer
                   */
                  memmove((char *)(end_boundary+strlen(disclaimertext)),end_boundary,
                                     (strlen(end_boundary) + 1)) ;
                  memcpy((char *)end_boundary,disclaimertext,strlen(disclaimertext));
                  priv->len += strlen(disclaimertext);
                  priv->txtdiscl_inserted=1;
                   RC=1;
                }
           }
        if (txtplain_p!=(char *)NULL && !priv->txtdiscl_inserted)
           { /* Test for second boundary of LOTUS-NOTES client.
              */
             txtplain_p+=13;         
             end_boundary=(char *)strstr(txtplain_p, BOUNDARY2);
             if (end_boundary!=(char *)NULL)
                { /* Insert the disclaimer
                   */
                  memmove((char *)(end_boundary+strlen(disclaimertext)),end_boundary,
                                     (strlen(end_boundary) + 1)) ;
                  memcpy((char *)end_boundary,disclaimertext,strlen(disclaimertext));
                  priv->len += strlen(disclaimertext);
                  priv->txtdiscl_inserted=1;
                   RC=1;
                }
           }
       }
    if (priv->internal_sender && priv->htmlallow && !priv->htmldiscl_inserted)
       {/* Insertion in first HTML segment of multipart message.
         */
        txthtml_p=(char *)priv->body;
        txthtml_p=(char *)nocase_strstr(txthtml_p,"Content-Type: text/html");
        if (txthtml_p!=(char *)NULL )
           { /* Second segment of  Outlook  (alternative html format)
              * For those mailers with HTML preference that forget
              * everything about their </BODY> and terminate with
              * </html> only(NS Communicator) we have to determine what
              * comes first. End of body or end of html.
              */
             memset(BOUNDARY,0X00,BND_SZ);
             txthtml_p+=13;         
             p3=(char *)nocase_strstr(txthtml_p, "</body>");
             p4=(char *)nocase_strstr(txthtml_p, "</html>");
             p5=(char *)nocase_strstr(txthtml_p, "</font>");  /* Incomplete html from Notes */
             end_boundary=(char *)NULL;
             if (p3!=(char *)NULL && p4!=(char *)NULL)
                { if (p3<p4)
                     { end_boundary=p3;
                     }
                  else
                     { end_boundary=p4;
                     }
                }
             if (p3==(char *)NULL)
                { end_boundary=p4;
                }
             if (p4==(char *)NULL)
                { end_boundary=p3;
                }
             if (p3==(char *)NULL && p4==(char *)NULL && p5!=(char *)NULL)
                { /* </font> as the end of a HTML message.
                   * Apparently Lotus-Notes does not only forget about <BODY> </BODY>
                   * and <HTML> </HTML>, but also on a lot of other items!
                   */ 
                    end_boundary=p5;
                }
             strcpy(DISCL2,"<H6><BR><HR>");
             strcat(DISCL2,disclaimertext);
             strcat(DISCL2,"<BR><HR><BR></H6>");
             if (end_boundary!=(char *)NULL)
                { /* Insert the disclaimer
                   */
                  memmove((char *)(end_boundary+strlen(DISCL2)),
                                   end_boundary, (strlen(end_boundary)+1));
                  memcpy((char *)end_boundary,DISCL2,strlen(DISCL2));
   	          priv->len += strlen(DISCL2);
                  priv->htmldiscl_inserted=1;
                  RC=1;
                }
           }
       }
    if ( priv->internal_sender  &&
         !priv->htmldiscl_inserted && !priv->txtdiscl_inserted &&
         (priv->msgtype==3 || priv->msgtype==1 || priv->msgtype==2))
       { /* Just one /text/plain message no attachments OUTLOOK-97 MIME.
          * We did not insert anything else before.
          * It is save to add the text disclaimer to the end of the message
          * body.
          */
        memcpy((char *)(priv->body+ priv->len), disclaimertext, strlen(disclaimertext));
        priv->len +=strlen(disclaimertext);
        RC=1;
       }  
    if ( priv->internal_sender  &&
         !priv->htmldiscl_inserted && !priv->txtdiscl_inserted &&
         priv->msgtype==4 ) 
       { /* Just one /text/html message no attachments.
          * We did not insert anything else before.
          */
        txthtml_p=(char *)priv->body;
        if (txthtml_p!=(char *)NULL )
           { /* Second segment of  Outlook  (alternative html format)
              * For those mailers with HTML preference that forget
              * everything about their </BODY> and terminate with
              * </html> only(NS Communicator) we have to determine what
              * comes first. End of body or end of html.
              */
             memset(BOUNDARY,0X00,BND_SZ);
             txthtml_p+=13;         
             end_boundary=(char *)NULL;
             p3=(char *)nocase_strstr(txthtml_p, "</body>");
             p4=(char *)nocase_strstr(txthtml_p, "</html>");
             if (p3!=(char *)NULL && p4!=(char *)NULL)
                {  /* Both tags present. Find the lowest position. 
                    */
                  if (p3<p4)
                     { end_boundary=p3;
                     }
                  else
                     { end_boundary=p4;
                     }
                }
             if (p3==(char *)NULL)
                { end_boundary=p4;
                }
             if (p4==(char *)NULL)
                { end_boundary=p3;
                }
             strcpy(DISCL2,"<H6><BR><HR>");
             strcat(DISCL2,disclaimertext);
             strcat(DISCL2,"<BR><HR><BR></H6>");
             tmp_p=(char *)nocase_strstr(DISCL2,"+******+");
             if (tmp_p!=(char *)NULL)
                { /* Substitute the marker with a ruler 
                   */                
                  memcpy( tmp_p,"<BR><HR>",8);
                }
             if (end_boundary!=(char *)NULL)
                { /* Insert the disclaimer
                   */
                  memmove((char *)(end_boundary+strlen(DISCL2)),
                                   end_boundary, (strlen(end_boundary)+1));
                  memcpy((char *)end_boundary,DISCL2,strlen(DISCL2));
   	          priv->len += strlen(DISCL2);
                  priv->htmldiscl_inserted=1;
                  RC=1;
                }
           }

       }  
    return(RC);
   }
/********************************************************/
void initDisclaimer(SMFICTX *ctx, char *envfrom)
{  /* Initializes the disclaimer and inserts a separator string. 
    */
  struct mlfiPriv *priv = MLFIPRIV;  

  priv->htmldiscl_inserted = 0;
  priv->txtdiscl_inserted = 0;
  priv->allow=0;
  priv->htmlallow=0;
  priv->msgtype =0;
  memset(priv->boundary,0X00,BND_SZ);
}
/***********************************************************/
void detectMsgFormat(SMFICTX *ctx, char *headerf, char *headerv) 
{ /* Tries to figure out what kind of mailer is being used
   * and what separator string is used.
   */ 
  struct mlfiPriv *priv = MLFIPRIV;  
  char *p1, *p2, *p3;
  if (!nocase_strncmp(headerf,"Content-Type",strlen("Content-Type"))
             && !priv->msgtype)
       {
        if ((p3=nocase_strstr(headerv,"multipart/")) !=(char *)NULL)
             { /* Allow  hanky-panky with the message-body.
                * Test for multipart message.
                * If so, allow insertion.
                * Determine the boundary-string for separate messages
                * in a mulipart MIME-message.
                */
              if ((p3=nocase_strstr(headerv,"multipart/alternative"))!=(char *)NULL)
                        { /*  Messages consisting of at least two identical
                           *  parts. (First part is plain/text, second is
                           *  the same message in HTML format.
                           */
                          priv->allow=1;
                          priv->htmlallow=1;
                          priv->msgtype=1;   
                        }
               if ((p3=nocase_strstr(headerv,"multipart/mixed"))!=(char *)NULL)
                        { /* Message consisting of multiple parts.
                           * First two parts need not contain the same message.
                           * The first chunk contains the message.
                           * Can be /text/plain or /text/html
                           */
                          priv->allow=1;
                          priv->htmlallow=1;
                          priv->msgtype=2;   
                         }
                      p1=nocase_strstr(headerv,"boundary=\"");
                      if (p1!=(char *)NULL)
                        { /*Quoted boundary 
                           */
                          p1+=strlen("boundary=\"");
                          p2=nocase_strstr((char *)p1,"\"");
                          if (p2!=(char *)NULL)
                             {/* Properly terminated by quote
                               */
                              memset(priv->boundary,0X00,BND_SZ);
                              strncpy(priv->boundary,p1,(int)(p2-p1));
                             }
                        }
                      else
                        { /* Unquoted boundary. Whats up next??
                           * Outlook Express 5.50.4133.2400
                           */
                          p1=nocase_strstr(headerv,"boundary=");
                          if (p1!=(char *)NULL)
                             {
                               p1+=strlen("boundary=");
                               p2=nocase_strstr((char *)p1,"\n");
                               if (p2==(char *)NULL)
                                  { p2=nocase_strstr((char *)p1,"\r");
                                  }
                               else
                                  {
                                    memset(priv->boundary,0X00,BND_SZ);
                                    strncpy(priv->boundary,p1,(int)(p2-p1));
                                  }
                             }
                        }
                   }
                if((p3=nocase_strstr(headerv,"text/plain"))!=(char *)NULL)
                   { /*
                      * Test for singular plain text message.
                      */
                      priv->msgtype=3;   
                   }
                if((p3=nocase_strstr(headerv,"text/html"))!=(char *)NULL)
                   { /*
                      * Test for singular message in HTML format.
                      */
                      priv->htmlallow=1;
                      priv->msgtype=4;   
                   }
           }
}
#endif
